@@ -62,17 +62,18 @@ src = lift go >>= yield >> src
 --
 cmd2task :: ConduitT DM.ProcSpawnCommand (IOTask ()) AppContext ()
 cmd2task = await >>= \case
-  Just cmd -> flip catchError errHdl $ do
+  Just cmd -> flip catchError (errHdl cmd) $ do
     lift (go cmd) >>= yield >> cmd2task
   Nothing -> do
     $logWarnS DM._LOGTAG "cmd2task: await returns nothing. skip."
     cmd2task
 
   where
-    errHdl :: String -> ConduitT DM.ProcSpawnCommand (IOTask ()) AppContext ()
-    errHdl msg = do
+    errHdl :: DM.ProcSpawnCommand -> String -> ConduitT DM.ProcSpawnCommand (IOTask ()) AppContext ()
+    errHdl procCmd msg = do
+      let jsonrpc = DM.getJsonRpcProcSpawnCommand procCmd
       $logWarnS DM._LOGTAG $ T.pack $ "cmd2task: exception occurred. skip. " ++ msg
-      lift $ errorToolsCallResponse $ "cmd2task: exception occurred. skip. " ++ msg
+      lift $ errorToolsCallResponse jsonrpc $ "cmd2task: exception occurred. skip. " ++ msg
       cmd2task
 
     go :: DM.ProcSpawnCommand -> AppContext (IOTask ())
@@ -138,7 +139,9 @@ genProcRunTask :: DM.ProcRunCommandData -> AppContext (IOTask ())
 genProcRunTask cmdDat = case cmdDat^.DM.nameProcRunCommandData of
   "proc-spawn"  -> genProcSpawn cmdDat
   "proc-ssh"    -> genProcSpawn cmdDat
-  "proc-telnet" -> genProcTelnet cmdDat
+  "proc-telnet" -> genProcSpawn cmdDat
+  "proc-winpty" -> genProcSpawn cmdDat
+  "proc-plink"  -> genProcSpawn cmdDat
   "proc-cmd"    -> genProcCMD cmdDat
   "proc-ps"     -> genProcPS cmdDat
   x -> throwError $ "genProcRunTask: unsupported command. " ++ x
@@ -151,10 +154,11 @@ genProcSpawn cmdDat = do
   let name     = cmdDat^.DM.nameProcRunCommandData
       argsBS   = DM.unRawJsonByteString $ cmdDat^.DM.argumentsProcRunCommandData
       tout     = DM._TIMEOUT_MICROSEC
+      addEnv   = [("ProgramData", "C:\\ProgramData"), ("SystemRoot", "C:\\Windows")] -- for windows ssh on claude code.
 
-  prompts <- view DM.promptsDomainData <$> lift ask
-  resQ <- view DM.responseQueueDomainData <$> lift ask
-  procMVar <- view processAppData <$> ask
+  prompts   <- view DM.promptsDomainData <$> lift ask
+  resQ      <- view DM.responseQueueDomainData <$> lift ask
+  procMVar  <- view processAppData <$> ask
   lockTMVar <- view lockAppData <$> ask
 
   (cmdTmp, argsArrayTmp, addPrompts)  <- getCommandArgs name argsBS
@@ -163,10 +167,11 @@ genProcSpawn cmdDat = do
 
   $logDebugS DM._LOGTAG $ T.pack $ "genProcRunTask: cmd. " ++ cmd ++ " " ++ show argsArray
  
-  return $ procSpawnTask cmdDat resQ procMVar lockTMVar cmd argsArray (prompts++addPrompts) tout
+  return $ procSpawnTask cmdDat resQ procMVar lockTMVar cmd argsArray addEnv (prompts++addPrompts) tout
 
   where
-    -- | Get command and arguments from the given name and arguments.
+    -- |
+    --
     getCommandArgs :: String -> BL.ByteString -> AppContext (String, [String], [String])
     getCommandArgs "proc-spawn" argsBS = do
       argsDat <- liftEither $ eitherDecode $ argsBS
@@ -177,13 +182,30 @@ genProcSpawn cmdDat = do
     getCommandArgs "proc-ssh" argsBS = do
       argsDat <- liftEither $ eitherDecode $ argsBS
       let argsArray0 = argsDat ^. argumentsProcStringArrayToolParams
-          argsArray = if "-tt" `elem` argsArray0 then argsArray0 else "-tt" : argsArray0
+          argsArray  = if "-tt" `elem` argsArray0 then argsArray0 else "-tt" : argsArray0
       return ("ssh", argsArray, [")?", "password:"])
+      -- return ("ssh", ["-o", "LogLevel=DEBUG", "-v"] ++ argsArray, [")?", "password:"])
 
     getCommandArgs "proc-telnet" argsBS = do
       argsDat <- liftEither $ eitherDecode $ argsBS
       let argsArray = argsDat^.argumentsProcStringArrayToolParams
-      return ("telnet", argsArray, ["login:", "Password:"])
+          winpty = "plink"
+--          winpty = "C:\\Progra~1\\Git\\usr\\bin\\winpty.exe"
+      return (winpty, "-telnet":argsArray, ["login:", "Password:"])
+
+    getCommandArgs "proc-winpty" argsBS = do
+      argsDat <- liftEither $ eitherDecode $ argsBS
+      let argsArray = argsDat^.argumentsProcStringArrayToolParams
+          winpty = "winpty"
+          -- winpty = "C:\\Progra~1\\Git\\usr\\bin\\winpty.exe"
+      return (winpty, argsArray, ["login:", "Password:", ")?", "password:"])
+
+    getCommandArgs "proc-plink" argsBS = do
+      argsDat <- liftEither $ eitherDecode $ argsBS
+      let argsArray = argsDat^.argumentsProcStringArrayToolParams
+          plink = "plink"
+          -- plink = "C:\\Progra~1\\PuTTY\\plink.exe"
+      return (plink, argsArray, ["login:", "Password:", ")?", "password:"])
 
     getCommandArgs x _ = throwError $ "getCommandArgs: unsupported command. " ++ x
 
@@ -196,10 +218,11 @@ procSpawnTask :: DM.ProcRunCommandData
               -> STM.TMVar ()
               -> String
               -> [String]
+              -> [(String, String)]
               -> [String]
               -> Int
               -> IOTask ()
-procSpawnTask cmdDat resQ procVar lockTMVar cmd args prompts tout = do
+procSpawnTask cmdDat resQ procVar lockTMVar cmd args addEnv prompts tout = do
   hPutStrLn stderr $ "[INFO] PMS.Infra.ProcSpawn.DS.Core.procSpawnTask start. "
 
   STM.atomically (STM.takeTMVar procVar) >>= \case
@@ -207,12 +230,14 @@ procSpawnTask cmdDat resQ procVar lockTMVar cmd args prompts tout = do
       STM.atomically $ STM.putTMVar procVar $ Just p
       hPutStrLn stderr "[ERROR] PMS.Infrastructure.DS.Core.procSpawnTask: pms is already connected."
       toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "process is already running."
-    Nothing -> flip E.catchAny errHdl $ runProc procVar cmd args
+    Nothing -> do
+      flip E.catchAny errHdl $ runProc procVar cmd args addEnv
 
   STM.atomically (STM.readTMVar procVar) >>= \case
-    Just p -> race (DM.expect lockTMVar (readProc p) prompts) (CC.threadDelay tout) >>= \case
-      Left res -> toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) ExitSuccess (maybe "Nothing" id res) ""
-      Right _  -> toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "timeout occurred."
+    Just p -> do
+      race (DM.expect lockTMVar (readProc p) prompts) (CC.threadDelay tout) >>= \case
+        Left res -> toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) ExitSuccess (maybe "Nothing" id res) ""
+        Right _  -> toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "timeout occurred."
     Nothing -> do
       hPutStrLn stderr "[ERROR] PMS.Infrastructure.DS.Core.procSpawnTask: unexpected. proc not found."
       toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "unexpected. proc not found."
@@ -259,7 +284,7 @@ genProcCMDTask cmdDat resQ procVar lockTMVar prompts tout = flip E.catchAny errH
       STM.atomically $ STM.putTMVar procVar $ Just p
       hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcCMDTask: process is already running."
       E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcCMDTask: process is already running."
-    Nothing -> runProc procVar "cmd" []
+    Nothing -> runProc procVar "cmd" [] []
 
   STM.atomically (STM.readTMVar procVar) >>= \case
     Nothing -> do
@@ -292,86 +317,6 @@ genProcCMDTask cmdDat resQ procVar lockTMVar prompts tout = flip E.catchAny errH
       toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" (show e)
 
 
-
--- |
---   
-genProcTelnet :: DM.ProcRunCommandData -> AppContext (IOTask ())
-genProcTelnet cmdDat = do
-  $logDebugS DM._LOGTAG $ T.pack $ "genProcTelnet: called. "
-
-  let tout = DM._TIMEOUT_MICROSEC
-
-  prompts <- view DM.promptsDomainData <$> lift ask
-  resQ <- view DM.responseQueueDomainData <$> lift ask
-  procMVar <- view processAppData <$> ask
-  lockTMVar <- view lockAppData <$> ask
-  
-  return $ genProcTelnetTask cmdDat resQ procMVar lockTMVar prompts tout
-
-
--- |
---   
-genProcTelnetTask :: DM.ProcRunCommandData
-               -> STM.TQueue DM.McpResponse
-               -> STM.TMVar (Maybe ProcData)
-               -> STM.TMVar ()
-               -> [String]
-               -> Int
-               -> IOTask ()
-genProcTelnetTask cmdDat resQ procVar lockTMVar prompts tout = flip E.catchAny errHdl $ do
-  hPutStrLn stderr $ "[INFO] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask start. "
-
-  STM.atomically (STM.takeTMVar procVar) >>= \case
-    Just p -> do
-      STM.atomically $ STM.putTMVar procVar $ Just p
-      hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask: process is already running."
-      E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask: process is already running."
-    Nothing -> runProc procVar "telnet" ["172.16.0.18"]
-
-  STM.atomically (STM.readTMVar procVar) >>= \case
-    Nothing -> do
-      hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask: process is not started."
-      E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask: process is not started."
-    Just procDat -> do
-      let wHdl = procDat^.wHdLProcData
-          msg  =  "phoityne"
-          cmd  = TE.encodeUtf8 $ T.pack $ msg ++ DM._LF
-      
-      hPutStrLn stderr $ "[INFO] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask username : " ++ BS8.unpack cmd
-      BS.hPut wHdl cmd
-      hFlush wHdl
-
-  STM.atomically (STM.readTMVar procVar) >>= \case
-    Nothing -> do
-      hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask: process is not started."
-      E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask: process is not started."
-    Just procDat -> do
-      let wHdl = procDat^.wHdLProcData
-          msg  =  "1qaz2wsx"
-          cmd  = TE.encodeUtf8 $ T.pack $ msg ++ DM._LF
-      
-      hPutStrLn stderr $ "[INFO] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask password : " ++ BS8.unpack cmd
-      BS.hPut wHdl cmd
-      hFlush wHdl
-
-  STM.atomically (STM.readTMVar procVar) >>= \case
-    Just p -> race (DM.expect lockTMVar (readProc p) prompts) (CC.threadDelay tout) >>= \case
-      Left res -> toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) ExitSuccess (maybe "Nothing" id res) ""
-      Right _  -> toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "timeout occurred."
-    Nothing -> do
-      hPutStrLn stderr "[ERROR] PMS.Infrastructure.DS.Core.genProcTelnetTask: unexpected. proc not found."
-      toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "unexpected. proc not found."
-
-  hPutStrLn stderr "[INFO] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask end."
-
-  where
-    errHdl :: E.SomeException -> IO ()
-    errHdl e = do
-      STM.atomically $ STM.putTMVar procVar Nothing
-      hPutStrLn stderr $ "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcTelnetTask: exception occurred. " ++ show e
-      toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" (show e)
-
-
 -- |
 --   
 genProcPS :: DM.ProcRunCommandData -> AppContext (IOTask ())
@@ -405,7 +350,7 @@ genProcPSTask cmdDat resQ procVar lockTMVar prompts tout = flip E.catchAny errHd
       STM.atomically $ STM.putTMVar procVar $ Just p
       hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcPSTask: process is already running."
       E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcPSTask: process is already running."
-    Nothing -> runProc procVar "powershell" ["-NoLogo", "-NoExit", "-Command", initCmd]
+    Nothing -> runProc procVar "powershell" ["-NoLogo", "-NoExit", "-Command", initCmd] []
 
   STM.atomically (STM.readTMVar procVar) >>= \case
     Just p -> race (DM.expect lockTMVar (readProc p) prompts) (CC.threadDelay tout) >>= \case
